@@ -8,7 +8,7 @@ use Module::Runtime qw(require_module);
 use Carp qw(confess croak);
 use List::Util qw(first);
 
-our $VERSION = '0.21';
+our $VERSION = '0.22';
 
 sub register_types {
   my ($type_definitions, $into, $moose_namespace) = @_;
@@ -38,19 +38,29 @@ sub make_type {
   my ($type_definition, $moose_namespace) = @_;
   my $test = $type_definition->{test};
 
-  my $full_test = $test;
   if (my $subtype_of = $type_definition->{subtype_of}) {
-    my $die_message =
-      "Must define a 'from' namespace for the parent type: $subtype_of when defining type: $type_definition->{name}";
-    my $from = $type_definition->{from}
-      || croak $die_message; ## no critic qw(ErrorHandling::RequireUseOfExceptions)
-    my $subtype_test = $from . '::is_' . $subtype_of;
-    no strict 'refs';    ## no critic qw(TestingAndDebugging::ProhibitNoStrict)
-    $full_test = sub { return (&{$subtype_test}(@_) && $test->(@_)); };
+    if (!ref $subtype_of) {
+      my $from = $type_definition->{from}
+        || croak "Must define a 'from' namespace for the parent type: $subtype_of when defining type: $type_definition->{name}";
+      $subtype_of = do {
+        no strict 'refs';
+        \&{$from . '::' . $subtype_of};
+      };
+    }
+    # Assume a (base) test always exists even if you must write: test => sub {1}
+    my $base_test = $test;
+    $test = sub {
+      my $value = shift;
+      local $@;
+      eval { $subtype_of->($value); 1 } or return;
+      # TODO implement: eval { $base_test->($value); 1 } paradigm
+      $base_test->($value) or return;
+      return 1;
+    };
   }
 
   my $isa = sub {
-    return if $full_test->(@_);
+    return if $test->(@_);
     local $Carp::Internal{"MooX::Types::MooseLike"} = 1;  ## no critic qw(Variables::ProhibitPackageVars)
     confess $type_definition->{message}->(@_) ;  ## no critic qw(ErrorHandling::RequireUseOfExceptions)
     };
@@ -125,7 +135,7 @@ sub make_type {
         return $isa;
       }
       },
-    is_type => sub { $full_test->(@_) },
+    is_type => sub { $test->(@_) },
     };
 }
 
@@ -219,6 +229,8 @@ Arthur Axel fREW Schmidt (cpan:FREW) <frioux@gmail.com>
 
 Toby Inkster (cpan:TOBYINK) <tobyink@cpan.org>
 
+Graham Knop (cpan:HAARG) <haarg@cpan.org>
+
 =head1 COPYRIGHT
 
 Copyright (c) 2011-2013 the MooX::Types::MooseLike L</AUTHOR> and
